// Copyright 2005 "Gilles Degottex"

// This file is part of "midingsolo"

// "midingsolo" is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// "midingsolo" is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "CustomMainForm.h"

#include <deque>
#include <iostream>
#include <cmath>
using namespace std;
#include <qapplication.h>
#include <qsettings.h>
#include <qlabel.h>
#include <qgl.h>
#include <qlayout.h>
#include <qtimer.h>
#include <qcheckbox.h>
#include <qaction.h>
#include <qtimer.h>
#include <qprogressbar.h>
#include <qradiobutton.h>
#include <qstring.h>
#include <qlcdnumber.h>
#include <qdial.h>
#include <qslider.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qtoolbox.h>
#include <qimage.h>
#include <qsplitter.h>
#include <qsizepolicy.h>
#include <qgroupbox.h>
#include <qstatusbar.h>
#include <qtextedit.h>
#include <qlineedit.h>
#include "res.h"

#include "ANR.h"

#include <Music/Music.h>
using namespace Music;

void CustomMainForm::refresh()
{
	if(m_reset)
	{
		initializeAlgos();
		m_reset = false;
	}

	if(anr().m_capture_thread.isCapturing())
		fill_buffer();

	if(m_incoming_data)
	{
		recognition();

		if(m_timer_update_views.elapsed()>40)	// 25 images/second
		{
			refresh_views();

			m_timer_update_views.start();
		}
	}
//	else cerr << "no data to eat !" << endl;
}

void CustomMainForm::refresh_views()
{
	if(ui_chkFollowRecentNote->isChecked() && m_selected_note!=0)
	{
		map<int,ANR::NoteDescription*>::iterator it=anr().m_notes_tag2descr.find(m_selected_note);

		if(it!=anr().m_notes_tag2descr.end())
		{
			m_glMonoGraph->m_selected_note = (*it).second;
//			m_glPolyGraph->m_selected_note = (*it).second;
		}
	}
	if(m_current_graph!=NULL)
		m_current_graph->updateGL();

	if(m_glComponents->isVisible())
	{
		m_glComponents->m_max_value = 1.0;
		m_glComponents->updateGL();
	}

	refresh_infos();
}

void CustomMainForm::fill_buffer()
{
	m_incoming_data = false;

	int limit = int( GetSamplingRate() * double(anr().getRefreshTime())/1000.0 );

	double limit_max_sec;
	if(anr().m_capture_thread.getPacketSize()==0)
		limit_max_sec = 0.1;
	else
		limit_max_sec = (ui_spinBufferFactor->value()/100.0)/(GetSamplingRate()/anr().m_capture_thread.getPacketSize());

//	cerr << "limit=" << limit << " limit_max_sec=" << limit_max_sec << endl;

	anr().m_capture_thread.lock();
	anr().m_nb_new_data = 0;
	m_new_data_amplitude = 0.0;
	while(!anr().m_capture_thread.m_values.empty() &&
			((int(anr().m_capture_thread.m_values.size()) > GetSamplingRate()*limit_max_sec) || (anr().m_nb_new_data<limit)))
	{
		m_incoming_data = true;

		m_new_data_amplitude = max(m_new_data_amplitude, fabs(anr().m_capture_thread.m_values.back()));

		anr().m_queue.push_front(anr().m_capture_thread.m_values.back());
		if(m_glMonoGraph!=NULL)
			m_glMonoGraph->m_pending_queue.push_front(anr().m_capture_thread.m_values.back());
		if(m_glPolyGraph!=NULL)
			m_glPolyGraph->m_pending_queue.push_front(anr().m_capture_thread.m_values.back());
		anr().m_capture_thread.m_values.pop_back();

		anr().m_nb_new_data++;
	}
	anr().m_capture_thread.unlock();
	m_glMonoGraph->add_maxs();
	m_glPolyGraph->add_maxs();

	// drop unused data TODO max_size adapté à la taille minimale
	int max_size = (m_current_graph->setting_spinDuration->value()*GetSamplingRate())/1000;
	while(int(anr().m_queue.size())>max_size)
		anr().m_queue.pop_back();
}

void CustomMainForm::recognition()
{
	anr().recognize();

	if(ui_chkFollowRecentNote->isChecked())
		m_selected_note = anr().m_most_recent_note;

//	cerr << m_current_graph->m_queue_amplitude << endl;
	ui_prVolume->setProgress(int(100*m_new_data_amplitude));

	if(ui_chkAutoSetVolumeMax->isChecked() &&
			m_new_data_amplitude > double(ui_slVolumeMax->value())/ui_slVolumeMax->maxValue())
		ui_slVolumeMax->setValue(int(m_new_data_amplitude*ui_slVolumeMax->maxValue()));
}

void CustomMainForm::refresh_infos()
{
//	cerr << "CustomMainForm::refresh_infos" << endl;
	
	// R-T
	QColor ok_color(83,165,105);
	QColor prb_color(180,74,74);

	QString real_time_status;

	ui_txtRealRefreshTime->setText(QString::number(anr().getRefreshTime())+" ms");
	if(anr().getRefreshTime() < ui_spinRefreshTime->value()*1.5)
		ui_lblRealRefreshStatus->setBackgroundColor(ok_color);
	else
	{
		ui_lblRealRefreshStatus->setBackgroundColor(prb_color);
		real_time_status += "- Your CPU is too slow [you should consider raising Refresh time]\n";
	}
//	cerr << "CustomMainForm::refresh_infos" << endl;

	ui_txtRefreshVariation->setText(QString::number(anr().m_refresh_variation)+" ms");
	if(anr().m_refresh_variation <= anr().getRefreshTime()/2)
		ui_lblRefreshVariationStatus->setBackgroundColor(ok_color);
	else
	{
		ui_lblRefreshVariationStatus->setBackgroundColor(prb_color);
		real_time_status += "- Real-time recognition is not stable enough [you should consider closing applications]\n";
	}
//	cerr << "CustomMainForm::refresh_infos" << endl;

	int pending_data_ms = 0;
	pending_data_ms += anr().m_avg_refresh;
	pending_data_ms += 1000*anr().m_max_pending_data/GetSamplingRate();
	ui_txtMinPendingData->setText(QString::number(anr().m_min_pending_data));
	if(anr().m_min_pending_data > 0)
		ui_lblMinPendingDataStatus->setBackgroundColor(ok_color);
	else
	{
		ui_lblMinPendingDataStatus->setBackgroundColor(prb_color);
		real_time_status += "- The recording audio system is not stable enough\n";
	}
//	cerr << "CustomMainForm::refresh_infos" << endl;

	if(real_time_status=="")
	{
		ui_txtCPUState->setText("Real-time status seems ok");
		ui_lblRealTimeStatus->setBackgroundColor(ok_color);
	}
	else
	{
		ui_txtCPUState->setText("Recording and analysis parameters requires too much resources for this machine");
		ui_lblRealTimeStatus->setBackgroundColor(prb_color);
//		ui_txtCPUState->setText(real_time_status);
	}
//	cerr << "CustomMainForm::refresh_infos" << endl;

	ui_lblRecordingLatency->setText(QString::number(pending_data_ms)+" ms");

	// algo
	ui_lblGeneralAlgoLatency->setText(QString::number(int(100.0*float(anr().getCurrentAlgorithm()->getAlgoLatency()))/100.0)+" ms");
//	cerr << "CustomMainForm::refresh_infos" << endl;

	// Quantizer
	ui_txtMinUsedRecon->setText(QString::number(anr().m_min_used_recon));
	ui_lblNbNoteBySecond->setText(QString::number(1000/ui_spinTolerance->value()));
	ui_lblQuantizerLatency->setText(QString::number(int(anr().m_quantizer.getLatency()))+" ms");

//	cerr << "CustomMainForm::refresh_infos HOP" << endl;
	// Info
	double latency = pending_data_ms;
	latency += anr().getCurrentAlgorithm()->getAlgoLatency();
	latency += anr().m_quantizer.getLatency();
	QString str_latency = QString::number(int(100*latency)/100.0)+" ms";
	ui_lblTBRecognitionLatency->setText(str_latency);

	if(ui_chkFollowRecentNote->isChecked())
		refresh_noteinfo();

//	cerr << "/CustomMainForm::refresh_infos" << endl;
}

// ------------------------ initialisation ----------------------

void CustomMainForm::initializeAlgos()
{
//	cerr << "CustomMainForm::initializeAlgos" << endl;

	// rebuild algos
	anr().init();

	// fill params
	if(anr().m_algo_multicorr!=NULL)
	{
		anr().m_algo_multicorr->setVolumeTreshold(ui_spinVolumeTreshold->value()/1000.0);
		anr().m_algo_multicorr->setLatencyFactor(ui_spinMulticorrLatencyFactor->value());
		anr().m_algo_multicorr->setTestComplexity(ui_spinMulticorrTestComplexity->value());
		anr().m_algo_multicorr->setComponentsTreshold(ui_spinMulticorrComponentsTreshold->value()/100.0);
	}
	if(anr().m_algo_autocorr!=NULL)
	{
		anr().m_algo_autocorr->setVolumeTreshold(ui_spinVolumeTreshold->value()/1000.0);
		anr().m_algo_autocorr->setNoiseThreshold(ui_spinAlgoAutocorrErrorNoiseThreshold->value()/1000.0);
	}
	if(anr().m_algo_bubble!=NULL)
	{
		anr().m_algo_bubble->setVolumeTreshold(ui_spinVolumeTreshold->value()/1000.0);
		// ...
	}

	recognition();
	refresh_views();
}

// -------------------------------- ctor -------------------------------

CustomMainForm::CustomMainForm()
{
	m_current_graph = NULL;
	m_reset = false;
	m_selected_note = 0;

//	QPixmap icon;
//	icon.loadFromData(g_icon_graph, sizeof(g_icon_graph), "PNG");
//	ui_tbConfig->setItemIconSet(0, icon);
//	icon.loadFromData(g_icon_filter, sizeof(g_icon_filter), "PNG");
//	ui_tbConfig->setItemIconSet(4, icon);
//	icon.loadFromData(g_icon_info, sizeof(g_icon_info), "PNG");	
//	ui_tbConfig->setItemIconSet(5, icon);

	// Config Form
	connect(m_config_form.buttonOk, SIGNAL(clicked()), this, SLOT(configure_ok()));
	connect(m_config_form.ui_btnRestoreFactorySettings, SIGNAL(clicked()), this, SLOT(restoreFactorySettings()));
#ifdef CAPTURE_ALSA
#else
	m_config_form.ui_rdbtnALSA->hide();
	m_config_form.ui_grpALSA->hide();
#endif
#ifdef CAPTURE_JACK
#else
	m_config_form.ui_rdbtnJACK->hide();
	m_config_form.ui_grpJACK->hide();
#endif
	connect(m_config_form.ui_btnAutoDetect, SIGNAL(clicked()), this, SLOT(autoDetectTransport()));
	connect(m_config_form.ui_rdbtnALSA, SIGNAL(clicked()), this, SLOT(alsaTransportSelected()));
	connect(m_config_form.ui_rdbtnJACK, SIGNAL(clicked()), this, SLOT(jackTransportSelected()));
	
	connect(m_config_form.ui_grpSTDOptions, SIGNAL(toggled(bool)), this, SLOT(STDOutputChanged(bool)));
	connect(m_config_form.ui_chkSTDGeneralNames, SIGNAL(toggled(bool)), this, SLOT(STDGeneralNamesChanged(bool)));
	connect(m_config_form.ui_chkSTDGeneralTransposition, SIGNAL(toggled(bool)), this, SLOT(STDGeneralTranspositionChanged(bool)));
	connect(m_config_form.ui_grpMIDIOptions, SIGNAL(toggled(bool)), this, SLOT(MIDIOutputChanged(bool)));
	connect(m_config_form.ui_txtMIDIClientName, SIGNAL(textChanged(const QString&)), this, SLOT(MIDIClientNameChanged(const QString&)));
	connect(m_config_form.ui_txtMIDIPortName, SIGNAL(textChanged(const QString&)), this, SLOT(MIDIPortNameChanged(const QString&)));


	connect(&(anr().m_capture_thread), SIGNAL(samplingRateChanged(int)), this, SLOT(samplingRateChanged(int)));
	connect(&(anr().m_capture_thread), SIGNAL(transportChanged(const QString&)), this, SLOT(transportChanged(const QString&)));
	connect(&(anr().m_capture_thread), SIGNAL(errorRaised(const QString&)), this, SLOT(errorRaised(const QString&)));

	m_glMonoGraph = new GLMonoGraph(centralWidget(), "Mono view graph");
	ui_layoutViews->addWidget(m_glMonoGraph);
	anr().m_quantizer.addListener(m_glMonoGraph);
	connect(m_config_form.ui_cbTonality, SIGNAL(highlighted(int)), m_glMonoGraph, SLOT(updateGL()));
	connect(m_config_form.ui_cbNotesName, SIGNAL(highlighted(int)), m_glMonoGraph, SLOT(updateGL()));
	connect(m_glMonoGraph, SIGNAL(selectedNoteChanged(int)), this, SLOT(selectedNoteChanged(int)));
	connect(m_glMonoGraph->setting_spinScaleFactor, SIGNAL(valueChanged(int)), ui_slVolumeMax, SLOT(setValue(int)));

	m_glPolyGraph = new GLPolyGraph(centralWidget(), "Poly view graph");
	ui_layoutViews->addWidget(m_glPolyGraph);
	anr().m_quantizer.addListener(m_glPolyGraph);
	m_glPolyGraph->hide();
	connect(m_config_form.ui_cbTonality, SIGNAL(highlighted(int)), m_glPolyGraph, SLOT(updateGL()));
	connect(m_config_form.ui_cbNotesName, SIGNAL(highlighted(int)), m_glPolyGraph, SLOT(updateGL()));
	connect(m_glPolyGraph, SIGNAL(selectedNoteChanged(int)), this, SLOT(selectedNoteChanged(int)));
	connect(m_glPolyGraph->setting_spinScaleFactor, SIGNAL(valueChanged(int)), ui_slVolumeMax, SLOT(setValue(int)));

	connect(m_glMonoGraph->setting_showWaveForm, SIGNAL(toggled(bool)), m_glPolyGraph->setting_showWaveForm, SLOT(setOn(bool)));
	connect(m_glPolyGraph->setting_showWaveForm, SIGNAL(toggled(bool)), m_glMonoGraph->setting_showWaveForm, SLOT(setOn(bool)));
	connect(m_glMonoGraph->setting_spinScaleFactor, SIGNAL(valueChanged(int)), m_glPolyGraph->setting_spinScaleFactor, SLOT(setValue(int)));
	connect(m_glPolyGraph->setting_spinScaleFactor, SIGNAL(valueChanged(int)), m_glMonoGraph->setting_spinScaleFactor, SLOT(setValue(int)));
	connect(m_glMonoGraph->setting_autoScale, SIGNAL(toggled(bool)), m_glPolyGraph->setting_autoScale, SLOT(setOn(bool)));
	connect(m_glPolyGraph->setting_autoScale, SIGNAL(toggled(bool)), m_glMonoGraph->setting_autoScale, SLOT(setOn(bool)));
	connect(m_glMonoGraph->setting_showStabilityGraph, SIGNAL(toggled(bool)), m_glPolyGraph->setting_showStabilityGraph, SLOT(setOn(bool)));
	connect(m_glPolyGraph->setting_showStabilityGraph, SIGNAL(toggled(bool)), m_glMonoGraph->setting_showStabilityGraph, SLOT(setOn(bool)));
	connect(m_glMonoGraph->setting_spinDuration, SIGNAL(valueChanged(int)), m_glPolyGraph->setting_spinDuration, SLOT(setValue(int)));
	connect(m_glPolyGraph->setting_spinDuration, SIGNAL(valueChanged(int)), m_glMonoGraph->setting_spinDuration, SLOT(setValue(int)));

	m_current_graph = m_glMonoGraph;

	// additional views
	m_glComponents = new GLComponents(centralWidget());
	ui_layoutViews->addWidget(m_glComponents);
	m_glComponents->setting_show->addTo(ui_tbViews);
	m_glComponents->setting_show->setOn(false);

//	QValueList<int> height_list;
//	height_list.push_front(height()/2);
//	height_list.push_front(height()/2);
//	ui_splitterViews->setSizes(height_list);

	m_timer_refresh = new QTimer(this, "m_timer_refresh");
	connect((QObject*)m_timer_refresh, SIGNAL(timeout()), this, SLOT(refresh()));

	m_settings.setPath("gna.org", "midingsolo");
	m_settings.beginGroup("/midingsolo/");
	m_settings.beginGroup("default/");
	loadSettings();
	
	if(m_config_form.ui_chkAutoDetect->isChecked())
		anr().m_capture_thread.autoDetectTransport();

	configure_ok();

//	m_timer_refresh->start(ui_spinRefreshTime->value());

	anr().start();
	m_timer_update_views.start();

//	cerr << "/CustomMainForm::CustomMainForm" << endl;
}

void CustomMainForm::transportChanged(const QString& name)
{
//	cerr << "CustomInstrumentTunerForm::transportChanged " << name << endl;

	if(name=="ALSA")
		alsaTransportSelected();
	if(name=="JACK")
		jackTransportSelected();
}
void CustomMainForm::autoDetectTransport()
{
	anr().m_capture_thread.autoDetectTransport();

	if(anr().m_capture_thread.getCurrentTransport()=="ALSA")
		alsaTransportSelected();
	if(anr().m_capture_thread.getCurrentTransport()=="JACK")
		jackTransportSelected();

	configure_ok();
}

void CustomMainForm::alsaTransportSelected()
{
	m_config_form.ui_rdbtnALSA->setChecked(true);
	m_config_form.ui_rdbtnJACK->setChecked(false);

	m_config_form.ui_grpALSA->setEnabled(m_config_form.ui_rdbtnALSA->isChecked());
	m_config_form.ui_grpJACK->setEnabled(m_config_form.ui_rdbtnJACK->isChecked());
}
void CustomMainForm::jackTransportSelected()
{
	m_config_form.ui_rdbtnJACK->setChecked(true);
	m_config_form.ui_rdbtnALSA->setChecked(false);

	m_config_form.ui_grpALSA->setEnabled(m_config_form.ui_rdbtnALSA->isChecked());
	m_config_form.ui_grpJACK->setEnabled(m_config_form.ui_rdbtnJACK->isChecked());
}

// be careful it's not always raised by the Qt thread !
// dont touch the QGLWidgets
void CustomMainForm::samplingRateChanged(int rate)
{
	if(m_config_form.ui_rdbtnJACK->isChecked())
		m_config_form.ui_lblJACKSamplingRate->setText(QString::number(rate));

	Music::SetSamplingRate(rate);

//	anr().m_capture_thread.lock();
//	anr().m_capture_thread.m_values.clear();
//	anr().m_queue.clear();
//	m_glMonoGraph->clear();
//	m_glPolyGraph->clear();
//	anr().m_capture_thread.unlock();

	m_reset = true;		// do like this because raised sometimes by the capture thread, so dont touch model through this fonction
}

void CustomMainForm::errorRaised(const QString& error)
{
	statusBar()->message(QString("ERROR: ")+error);

//	QMessageBox::warning(this, "Sound Capture", QString("Bad source: ")+src, QMessageBox::Ok, QMessageBox::NoButton);	//TODO crash ! thread of this signal is not the same as the one used for the window.
}

void CustomMainForm::resizeEvent(QResizeEvent *)
{
}

void CustomMainForm::graphMonoViewToggled(bool on)
{
	if(graphMonoViewAction->isOn())
	{
		m_current_graph = m_glMonoGraph;
		m_glMonoGraph->show();
	}
	else
		m_glMonoGraph->hide();
}
void CustomMainForm::graphPolyViewToggled(bool on)
{
	if(graphPolyViewAction->isOn())
	{
		m_current_graph = m_glPolyGraph;
		m_glPolyGraph->show();
	}
	else
		m_glPolyGraph->hide();
}

void CustomMainForm::pause(bool toggled)
{
	if(toggled)	
	{
		m_timer_refresh->stop();
		refresh_views();
	}
	else
		m_timer_refresh->start(ui_spinRefreshTime->value());

	anr().pause(toggled);
}
	
void CustomMainForm::selectedNoteChanged(int tag)
{
	m_selected_note = tag;

	refresh_noteinfo();
}

void CustomMainForm::refresh_noteinfo()
{
	if(m_selected_note!=0)
	{
		map<int,ANR::NoteDescription*>::iterator it=anr().m_notes_tag2descr.find(m_selected_note);

		if(it!=anr().m_notes_tag2descr.end())
		{
			ANR::NoteDescription* note = (*it).second;

			QString note_name = h2n(note->ht).c_str();

			ui_lblNotePitch->setText(note_name+" ("+QString::number(note->ht)+", "+QString::number(int(h2f(note->ht)*100)/100.0)+"hz)");

			ui_lblNoteDuration->setText(QString::number(int(note->duration)/1000.0)+" s");

			ui_lblNoteRecognitionsUsed->setText(QString::number(note->used_recons.size()));

			ui_lblNoteReliability->setText(QString::number(int(note->reliability*100))+"%");
		}
	}
}

void CustomMainForm::spinRefreshTimeChanged(int value)
{
	m_timer_refresh->changeInterval(value);
}

// set general params
void CustomMainForm::spinVolumeTresholdChanged(int value)
{
//	cerr << "CustomMainForm::spinVolumeTresholdChanged " << value << endl;

	m_glMonoGraph->m_treshold = value/1000.0;
	m_glPolyGraph->m_treshold = value/1000.0;

	if(anr().getCurrentAlgorithm()!=NULL)
		anr().getCurrentAlgorithm()->setVolumeTreshold(value/1000.0);
}
void CustomMainForm::spinAFreqChanged(int AFreq)
{
	if(AFreq!=Music::GetAFreq())
		Music::SetAFreq(AFreq);
}
void CustomMainForm::semitoneBoundsChanged()
{
//	cerr << "CustomMainForm::spinMinHTChanged " << value << endl;
	ui_spinMaxHT->setMinValue(ui_spinMinHT->value()+1);
	ui_spinMinHT->setMaxValue(ui_spinMaxHT->value()-1);
	ui_txtMinHT->setText(h2n(ui_spinMinHT->value()));
	ui_txtMaxHT->setText(h2n(ui_spinMaxHT->value()));

	Music::SetSemitoneBounds(ui_spinMinHT->value(), ui_spinMaxHT->value());
}

// select algo
void CustomMainForm::algoMultiCorrSelected()
{
	ui_btnAutoCorr->setChecked(false);
	ui_btnBubble->setChecked(false);

	anr().m_algo_current = anr().m_algo_multicorr;
}
void CustomMainForm::algoAutoCorrSelected()
{
	ui_btnMultiCorr->setChecked(false);
	ui_btnBubble->setChecked(false);
	
	anr().m_algo_current = anr().m_algo_autocorr;
}
void CustomMainForm::algoBubbleSelected()
{
	ui_btnMultiCorr->setChecked(false);
	ui_btnAutoCorr->setChecked(false);
	
	anr().m_algo_current = anr().m_algo_bubble;
}

// set specific algo params
void CustomMainForm::spinAlgoMulticorrLatencyFactorChanged(int value){
	if(anr().m_algo_multicorr!=NULL)	anr().m_algo_multicorr->setLatencyFactor(value);
}
void CustomMainForm::spinAlgoMulticorrTestComplexityChanged(int value){
	if(anr().m_algo_multicorr!=NULL)	anr().m_algo_multicorr->setTestComplexity(value);
}
void CustomMainForm::spinAlgoMulticorrComponentsTresholdChanged(int value){
	if(anr().m_algo_multicorr!=NULL)	anr().m_algo_multicorr->setComponentsTreshold(value/100.0);
}
void CustomMainForm::spinAlgoAutocorrErrorNoiseThresholdChanged(int value){
	if(anr().m_algo_autocorr!=NULL)		anr().m_algo_autocorr->setNoiseThreshold(value/1000.0);
}
void CustomMainForm::spinAlgoBubbleErrorNoiseThresholdChanged(int value){
	if(anr().m_algo_bubble!=NULL)		anr().m_algo_bubble->setErrorThreshold(value/100.0);
}
void CustomMainForm::spinAlgoBubbleConvolutionThresholdChanged(int value){
	if(anr().m_algo_bubble!=NULL)		anr().m_algo_bubble->setConvolutionThreshold(value/100.0);
}

// quantizer
void CustomMainForm::spinToleranceChanged(int value)
{
	anr().m_quantizer.setTolerance(value);
}
void CustomMainForm::spinMinDensityChanged(int value)
{
	anr().m_quantizer.setMinDensity(value/100.0);
}

// configure panel
void CustomMainForm::cbTonalityChanged(int index)
{
	switch(index)
	{
	  case 0: Music::SetTonality(0); break;
	  case 1: Music::SetTonality(+2); break;
	  case 2: Music::SetTonality(-3); break;
	}

	ui_txtMinHT->setText(h2n(ui_spinMinHT->value()));
	ui_txtMaxHT->setText(h2n(ui_spinMaxHT->value()));

//	cerr << "CustomMainForm::cbTonalityChanged " << Music::GetTonality() << endl;
}

void CustomMainForm::cbNotesNameChanged(int index)
{
	switch(index)
	{
	  case 0: Music::SetNotesName(LOCAL_ANGLO); break;
	  case 1: Music::SetNotesName(LOCAL_LATIN); break;
	}

	ui_txtMinHT->setText(h2n(ui_spinMinHT->value()));
	ui_txtMaxHT->setText(h2n(ui_spinMaxHT->value()));

//	cerr << "CustomMainForm::cbNotesNameChanged " << Music::GetNotesName() <<  endl;
}

void CustomMainForm::STDOutputChanged(bool enable)
{
//	cerr << "CustomMainForm::STDOutputChanged" << endl;
	anr().m_std_enabled = enable;
}
void CustomMainForm::STDGeneralNamesChanged(bool use_general)
{
//	cerr << "CustomMainForm::STDGeneralNamesChanged" << endl;
	anr().m_std_anglo_names = !use_general;
}
void CustomMainForm::STDGeneralTranspositionChanged(bool use_general)
{
//	cerr << "CustomMainForm::STDGeneralTranspositionChanged" << endl;
	anr().m_std_transpose = use_general;
}
void CustomMainForm::MIDIOutputChanged(bool enable)
{
//	cerr << "CustomMainForm::MIDIOutputChanged" << endl;
	anr().m_midi_enabled = enable;
}
void CustomMainForm::MIDIClientNameChanged(const QString& client_name)
{
	// TODO
}
void CustomMainForm::MIDIPortNameChanged(const QString& port_name)
{
	// TODO
}

void CustomMainForm::slVolumeMaxChanged(int value)
{
	if(!m_current_graph->setting_autoScale->isOn())
		m_current_graph->setting_spinScaleFactor->setValue(ui_slVolumeMax->value());

	m_current_graph->updateGL();
}

void CustomMainForm::slComponentMaxChanged(int value)
{
	//	cerr << "CustomMainForm::slComponentMaxChanged" << endl;
	if(m_glComponents->isVisible())
	{
		m_glComponents->m_max_value = 1.0;
		m_glComponents->updateGL();
	}
}

void CustomMainForm::saveSettings()
{
	// Sound Capture
	m_settings.writeEntry("ui_chkAutoDetect", m_config_form.ui_chkAutoDetect->isChecked()?1:0);
	m_settings.writeEntry("ui_rdbtnALSA", m_config_form.ui_rdbtnALSA->isChecked()?1:0);
	m_settings.writeEntry("ui_rdbtnJACK", m_config_form.ui_rdbtnJACK->isChecked()?1:0);
#ifdef CAPTURE_ALSA
	m_settings.writeEntry("ui_txtALSAPCMName", m_config_form.ui_txtALSAPCMName->text());
	m_settings.writeEntry("ui_chkALSASamplingRateMax", (m_config_form.ui_chkALSASamplingRateMax->isChecked()?1:0));
	m_settings.writeEntry("ui_spinALSASamplingRate", m_config_form.ui_spinALSASamplingRate->value());
#endif
#ifdef CAPTURE_JACK
	m_settings.writeEntry("ui_chkJACKAutoConnect", (m_config_form.ui_chkJACKAutoConnect->isChecked()?1:0));
	m_settings.writeEntry("ui_txtJACKSourcePort", m_config_form.ui_txtJACKSourcePort->text());
#endif
	
	// STD group options
	m_settings.writeEntry("ui_grpSTDOptions", m_config_form.ui_grpSTDOptions->isChecked());
	m_settings.writeEntry("ui_chkSTDGeneralNames", m_config_form.ui_chkSTDGeneralNames->isChecked());
	m_settings.writeEntry("ui_chkSTDGeneralTransposition", m_config_form.ui_chkSTDGeneralTransposition->isChecked());

	// MIDI group options
	m_settings.writeEntry("ui_grpMIDIOptions", m_config_form.ui_grpMIDIOptions->isChecked());
	m_settings.writeEntry("ui_txtMIDIClientName", m_config_form.ui_txtMIDIClientName->text());
	m_settings.writeEntry("ui_txtMIDIPortName", m_config_form.ui_txtMIDIPortName->text());

	// capture tuning
	m_settings.writeEntry("ui_spinRefreshTime", ui_spinRefreshTime->value());
	m_settings.writeEntry("ui_spinBufferFactor", ui_spinBufferFactor->value());

	m_settings.writeEntry("ui_spinVolumeTreshold", ui_spinVolumeTreshold->value());
	m_settings.writeEntry("ui_spinAFreq", ui_spinAFreq->value());
	m_settings.writeEntry("ui_spinMinHT", ui_spinMinHT->value());
	m_settings.writeEntry("ui_spinMaxHT", ui_spinMaxHT->value());

	// Transform
	m_settings.writeEntry("ui_spinAFreq", ui_spinAFreq->value());

	m_settings.writeEntry("ui_chkFollowRecentNote", ui_chkFollowRecentNote->isChecked());

	// algo
	m_settings.writeEntry("ui_spinMulticorrLatencyFactor", ui_spinMulticorrLatencyFactor->value());
	m_settings.writeEntry("ui_spinMulticorrTestComplexity", ui_spinMulticorrTestComplexity->value());
	m_settings.writeEntry("ui_spinMulticorrComponentsTreshold", ui_spinMulticorrComponentsTreshold->value());
	m_settings.writeEntry("ui_spinAlgoAutocorrErrorNoiseThreshold", ui_spinAlgoAutocorrErrorNoiseThreshold->value());
	m_settings.writeEntry("ui_spinAlgoBubbleErrorNoiseThreshold", ui_spinAlgoBubbleErrorNoiseThreshold->value());
	m_settings.writeEntry("ui_spinAlgoBubbleConvolutionThreshold", ui_spinAlgoBubbleConvolutionThreshold->value());

	// quantizer
	m_settings.writeEntry("ui_spinTolerance", ui_spinTolerance->value());
	m_settings.writeEntry("ui_spinMinDensity", ui_spinMinDensity->value());

	// information
	m_settings.writeEntry("ui_cbTonality", m_config_form.ui_cbTonality->currentItem());
	m_settings.writeEntry("ui_cbNotesName", m_config_form.ui_cbNotesName->currentItem());
}

void CustomMainForm::loadSettings()
{
//	cerr <<"CustomMainForm::loadSettings"<<endl;

	// Sound Capture
	m_config_form.ui_chkAutoDetect->setChecked(m_settings.readNumEntry("ui_chkAutoDetect", (m_config_form.ui_chkAutoDetect->isChecked()?1:0)));
	m_config_form.ui_rdbtnALSA->setChecked(m_settings.readNumEntry("ui_rdbtnALSA", (m_config_form.ui_rdbtnALSA->isChecked()?1:0)));
	m_config_form.ui_rdbtnJACK->setChecked(m_settings.readNumEntry("ui_rdbtnJACK", (m_config_form.ui_rdbtnJACK->isChecked()?1:0)));
#ifdef CAPTURE_ALSA
	m_config_form.ui_txtALSAPCMName->setText(m_settings.readEntry("ui_txtALSAPCMName", (m_config_form.ui_txtALSAPCMName->text())));
	m_config_form.ui_chkALSASamplingRateMax->setChecked(m_settings.readNumEntry("ui_chkALSASamplingRateMax", (m_config_form.ui_chkALSASamplingRateMax->isChecked()?1:0)));
	m_config_form.ui_spinALSASamplingRate->setValue(m_settings.readNumEntry("ui_spinALSASamplingRate", m_config_form.ui_spinALSASamplingRate->value()));
#endif
#ifdef CAPTURE_JACK
//	cout << m_config_form.ui_chkJACKAutoConnect->isChecked() << ":" << m_settings.readNumEntry("ui_chkJACKAutoConnect", 0) << endl;
//	cout << m_config_form.ui_txtJACKSourcePort->text() << ":" << m_settings.readEntry("ui_txtJACKSourcePort", "prout") << endl;
	m_config_form.ui_chkJACKAutoConnect->setChecked(m_settings.readNumEntry("ui_chkJACKAutoConnect", (m_config_form.ui_chkJACKAutoConnect->isChecked()?1:0)));
	m_config_form.ui_txtJACKSourcePort->setText(m_settings.readEntry("ui_txtJACKSourcePort", m_config_form.ui_txtJACKSourcePort->text()));
#endif

	// STD group options
	m_config_form.ui_grpSTDOptions->setChecked(m_settings.readBoolEntry("ui_grpSTDOptions", m_config_form.ui_grpSTDOptions->isChecked()));
	m_config_form.ui_chkSTDGeneralNames->setChecked(m_settings.readBoolEntry("ui_chkSTDGeneralNames", m_config_form.ui_chkSTDGeneralNames->isChecked()));
	m_config_form.ui_chkSTDGeneralTransposition->setChecked(m_settings.readBoolEntry("ui_chkSTDGeneralTransposition", m_config_form.ui_chkSTDGeneralTransposition->isChecked()));

	// MIDI group options
	m_config_form.ui_grpMIDIOptions->setChecked(m_settings.readBoolEntry("ui_grpMIDIOptions", m_config_form.ui_grpMIDIOptions->isChecked()));
	m_config_form.ui_txtMIDIClientName->setText(m_settings.readEntry("ui_txtMIDIClientName", m_config_form.ui_txtMIDIClientName->text()));
	m_config_form.ui_txtMIDIPortName->setText(m_settings.readEntry("ui_txtMIDIPortName", m_config_form.ui_txtMIDIPortName->text()));

	// Capture tuning
	ui_spinRefreshTime->setValue(m_settings.readNumEntry("ui_spinRefreshTime", ui_spinRefreshTime->value()));
	spinRefreshTimeChanged(ui_spinRefreshTime->value());

	ui_spinBufferFactor->setValue(m_settings.readNumEntry("ui_spinBufferFactor", ui_spinBufferFactor->value()));

	// Transform
	ui_spinAFreq->setValue(m_settings.readNumEntry("ui_spinAFreq", ui_spinAFreq->value()));
	Music::SetAFreq(ui_spinAFreq->value());
	ui_spinMinHT->setValue(m_settings.readNumEntry("ui_spinMinHT", ui_spinMinHT->value()));
	ui_spinMaxHT->setValue(m_settings.readNumEntry("ui_spinMaxHT", ui_spinMaxHT->value()));
	Music::SetSemitoneBounds(ui_spinMinHT->value(), ui_spinMaxHT->value());
	ui_chkFollowRecentNote->setChecked(m_settings.readBoolEntry("ui_chkFollowRecentNote", ui_chkFollowRecentNote->isChecked()));

	// algo
	ui_spinVolumeTreshold->setValue(m_settings.readNumEntry("ui_spinVolumeTreshold", ui_spinVolumeTreshold->value()));
	spinVolumeTresholdChanged(ui_spinVolumeTreshold->value());

	ui_spinMulticorrLatencyFactor->setValue(m_settings.readNumEntry("ui_spinMulticorrLatencyFactor", 1));
	spinAlgoMulticorrLatencyFactorChanged(ui_spinMulticorrLatencyFactor->value());
	ui_spinMulticorrTestComplexity->setValue(m_settings.readNumEntry("ui_spinMulticorrTestComplexity", 16));
	spinAlgoMulticorrTestComplexityChanged(ui_spinMulticorrTestComplexity->value());
	ui_spinMulticorrComponentsTreshold->setValue(m_settings.readNumEntry("ui_spinMulticorrComponentsTreshold", 33));
	spinAlgoMulticorrComponentsTresholdChanged(ui_spinMulticorrComponentsTreshold->value());
	ui_spinAlgoAutocorrErrorNoiseThreshold->setValue(m_settings.readNumEntry("ui_spinAlgoAutocorrErrorNoiseThreshold", 100));
	spinAlgoAutocorrErrorNoiseThresholdChanged(ui_spinAlgoAutocorrErrorNoiseThreshold->value());
	ui_spinAlgoBubbleErrorNoiseThreshold->setValue(m_settings.readNumEntry("ui_spinAlgoBubbleErrorNoiseThreshold", 10));
	spinAlgoBubbleErrorNoiseThresholdChanged(ui_spinAlgoAutocorrErrorNoiseThreshold->value());
	ui_spinAlgoBubbleConvolutionThreshold->setValue(m_settings.readNumEntry("ui_spinAlgoBubbleConvolutionThreshold", 10));
	spinAlgoBubbleConvolutionThresholdChanged(ui_spinAlgoBubbleConvolutionThreshold->value());

	// quantizer
	ui_spinTolerance->setValue(m_settings.readNumEntry("ui_spinTolerance", ui_spinTolerance->value()));
	spinToleranceChanged(ui_spinTolerance->value());
	ui_spinMinDensity->setValue(m_settings.readNumEntry("ui_spinMinDensity", ui_spinMinDensity->value()));
	spinMinDensityChanged(ui_spinMinDensity->value());

	// information
	m_config_form.ui_cbTonality->setCurrentItem(m_settings.readNumEntry("ui_cbTonality", m_config_form.ui_cbTonality->currentItem()));
	cbTonalityChanged(m_config_form.ui_cbTonality->currentItem());
	m_config_form.ui_cbNotesName->setCurrentItem(m_settings.readNumEntry("ui_cbNotesName", m_config_form.ui_cbNotesName->currentItem()));
	cbNotesNameChanged(m_config_form.ui_cbNotesName->currentItem());
}

void CustomMainForm::configure()
{
	m_config_form.show();
}
void CustomMainForm::configure_ok()
{
//	cerr << "CustomMainForm::configure_ok" << endl;

	if(m_config_form.ui_cbTonality->currentItem()==0)		SetTonality(0);
	else if(m_config_form.ui_cbTonality->currentItem()==1)	SetTonality(+2);
	else													SetTonality(-3);

	if(m_config_form.ui_cbNotesName->currentItem()==0)		SetNotesName(LOCAL_ANGLO);
	else													SetNotesName(LOCAL_LATIN);

#ifdef CAPTURE_ALSA
	if(m_config_form.ui_rdbtnALSA->isChecked())
	{
		anr().m_capture_thread.selectTransport("ALSA");
		m_config_form.ui_grpALSA->setEnabled(true);
		anr().m_capture_thread.setSource(m_config_form.ui_txtALSAPCMName->text());
		anr().m_capture_thread.stopCapture();			// have to do that or a bug appears
		if(m_config_form.ui_chkALSASamplingRateMax->isChecked())
			anr().m_capture_thread.setSamplingRate(CaptureThread::SAMPLING_RATE_MAX);
		else
			anr().m_capture_thread.setSamplingRate(m_config_form.ui_spinALSASamplingRate->value());
		anr().m_capture_thread.startCapture();
	}
	else
		m_config_form.ui_grpALSA->setDisabled(true);
#endif
#ifdef CAPTURE_JACK
	if(m_config_form.ui_rdbtnJACK->isChecked())
	{
		anr().m_capture_thread.selectTransport("JACK");
		m_config_form.ui_grpJACK->setEnabled(true);
		if(m_config_form.ui_chkJACKAutoConnect->isChecked())
			anr().m_capture_thread.setSource(m_config_form.ui_txtJACKSourcePort->text());
		else
			anr().m_capture_thread.setSource("");
		m_config_form.ui_lblJACKSamplingRate->setText(QString::number(anr().m_capture_thread.getSamplingRate()));
	}
	else
		m_config_form.ui_grpJACK->setDisabled(true);
#endif
	if(!anr().m_capture_thread.isCapturing())
		anr().m_capture_thread.reset();
}

void CustomMainForm::helpAbout()
{
	QMessageBox::about(this, "About Free Audio Note Recognition", QString("<h3>Version ")+PACKAGE_VERSION+"</h3><p><h3>Summary:</h3><p>This project aims to do a real-time audio note recognition software for monophonic signals like solos.<p><h3>Website:</h3><p>homepage: <a href=\"http://home.gna.org/midingsolo\">http://home.gna.org/midingsolo</a><p>development site: <a href=\"http://gna.org/projects/midingsolo\">http://gna.org/projects/midingsolo</a><p><h3>Authors:</h3><p>Gilles Degottex: gilles.degottex@net2000.ch");
}

CustomMainForm::~CustomMainForm()
{
	anr().m_capture_thread.stopCapture();
}

