// Copyright 2005 "Gilles Degottex"

// This file is part of "midingsolo"

// "midingsolo" is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// "midingsolo" is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

struct CustomMainForm;

#ifndef _CustomMainForm_h_
#define _CustomMainForm_h_

#include <qdatetime.h>
#include <qsettings.h>

#include "MainForm.h"
#include "ConfigForm.h"
#include "GLRTGraph.h"
#include "GLMonoGraph.h"
#include "GLPolyGraph.h"
#include "GLComponents.h"

struct CustomMainForm : MainForm
{
	GLMonoGraph* m_glMonoGraph;
	GLPolyGraph* m_glPolyGraph;
	GLRTGraph* m_current_graph;
	GLComponents* m_glComponents;

	QTimer* m_timer_refresh;

	QTime m_timer_update_views;

	virtual void resizeEvent(QResizeEvent *);

	void cbTonalityChanged(int index);
	void cbNotesNameChanged(int index);
	void slVolumeMaxChanged(int value);
	void slComponentMaxChanged(int value);
	void saveSettings();
	void loadSettings();
	void graphMonoViewToggled(bool on);
	void graphPolyViewToggled(bool on);
	void selectedNoteChanged(int tag);
	
	void spinRefreshTimeChanged(int value);

	void spinVolumeTresholdChanged(int value);
	void spinAFreqChanged(int AFreq);
	void semitoneBoundsChanged();
	void spinToleranceChanged(int value);
	void spinMinDensityChanged(int value);

	void algoMultiCorrSelected();
	void algoAutoCorrSelected();
	void algoBubbleSelected();
	void spinAlgoMulticorrLatencyFactorChanged(int value);
	void spinAlgoMulticorrTestComplexityChanged(int value);
	void spinAlgoMulticorrComponentsTresholdChanged(int value);
	void spinAlgoAutocorrErrorNoiseThresholdChanged(int value);
	void spinAlgoBubbleErrorNoiseThresholdChanged(int value);
	void spinAlgoBubbleConvolutionThresholdChanged(int value);

	// Config Form
	virtual void transportChanged(const QString& name);
	virtual void autoDetectTransport();
	virtual void alsaTransportSelected();
	virtual void jackTransportSelected();
	virtual void STDOutputChanged(bool);
	virtual void STDGeneralNamesChanged(bool);
	virtual void STDGeneralTranspositionChanged(bool);
	virtual void MIDIOutputChanged(bool);
	virtual void MIDIClientNameChanged(const QString&);
	virtual void MIDIPortNameChanged(const QString&);

	// CaptureThread signals
	virtual void samplingRateChanged(int rate);
	virtual void errorRaised(const QString& error);

	void pause(bool toggled);

	void helpAbout();

	ConfigForm m_config_form;
	void configure();
	void configure_ok();
	
	int m_selected_note;
	void refresh_noteinfo();

	void refresh();
	bool m_incoming_data;
	double m_new_data_amplitude;
	void fill_buffer();
	void recognition();
	void refresh_views();
	void refresh_infos();

	// ------------------------ initialisation ----------------------

	bool m_reset;
	void initializeAlgos();

	QSettings m_settings;
	bool m_settings_loaded;

	CustomMainForm();

	virtual ~CustomMainForm();
};

#endif // _CustomMainForm_h_
