// Copyright 2004 "Gilles Degottex"

// This file is part of "Music"

// "Music" is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// "Music" is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "GLComponents.h"
#include <iostream>
#include <GL/glut.h>
#include <CppAddons/StringAddons.h>
#include <Music/Music.h>
using namespace Music;
#include <qimage.h>
#include "ANR.h"
#include "res.h"

GLComponents::GLComponents(QWidget* parent)
: QGLWidget(parent, "GLComponents")
, View("Components", this)
, m_max_value(1.0)
{
	m_selected_st = Music::UNDEFINED_SEMITONE;

	// settings
	QImage img;
	img.loadFromData(g_icon_filter, sizeof(g_icon_filter), "PNG");
	setting_show->setIconSet(QIconSet(QImage(img)));
	setting_show->setOn(true);

	setting_autoScale = new QAction(this);
	setting_autoScale->setMenuText(tr("Auto scale"));
	setting_autoScale->setToggleAction(true);
	connect(setting_autoScale, SIGNAL(toggled(bool)), this, SLOT(update()));
	setting_autoScale->setOn(false);
	setting_autoScale->addTo(&m_popup_menu);
}

void GLComponents::mouseReleaseEvent(QMouseEvent* e)
{
	View::mouseReleaseEvent(e);

	if(e->button()==Qt::LeftButton)
	{
		float step = float(width())/GetNbSemitones();

		int scale_height = 12;

		int space = (step>2)?1:0;

		int selected_st = Music::UNDEFINED_SEMITONE;

		// vertical lines
		for(size_t i=0; selected_st==Music::UNDEFINED_SEMITONE && i<Music::GetNbSemitones(); i++)
		{
			int x_left = int(i*step);
			int x_right = int((i+1)*step)-space;

			if(x_left<e->x() && e->x()<x_right)
				selected_st = i;
		}

		if(selected_st!=Music::UNDEFINED_SEMITONE)
			selected_st+=Music::GetSemitoneMin();

		if(selected_st != m_selected_st)
		{
			m_selected_st = selected_st;
			emit(selectedSemitoneChanged(m_selected_st));
		}

		updateGL();
	}
}

void GLComponents::initializeGL()
{
	// Set the clear color to black
	glClearColor(1.0, 1.0, 1.0, 0.0);

	// glShadeModel( GL_FLAT );
	glShadeModel( GL_SMOOTH );

	glLoadIdentity();
}

void GLComponents::paintGL()
{
	glClear(GL_COLOR_BUFFER_BIT);

	// name
	string str = tr("Components");
	glColor3f(0.75,0.75,0.75);
	glRasterPos2i(2, height()-14);
	for(size_t i = 0; i < str.size(); i++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, str[i]);

	float step = float(width())/GetNbSemitones();

	if(anr().getCurrentTransform()!=NULL)
	{
		int scale_height = 12;

		double scale_factor = 1.0;
		if(setting_autoScale->isOn())	scale_factor = 1.0/anr().getCurrentTransform()->getComponentsMax();
		else							scale_factor = 1.0/m_max_value;

//		cerr<<"show bars auto="<<m_auto_scale<<" max="<<anr().getCurrentTransform()->getComponentsMax()<<" "<<m_auto_scale<<endl;
		int space = (step>2)?1:0;

		// vertical lines
		glBegin(GL_LINES);
		glColor3f(0.8, 0.8, 0.8);
		for(size_t i=1; i<GetNbSemitones(); i++)
		{
			int x = int(i*step);

			glVertex2i(x, scale_height);
			glVertex2i(x, height());
		}
		glEnd();

		// bars
		glBegin(GL_QUADS);
		for(size_t i=0; i<anr().getCurrentTransform()->getComponents().size(); i++)
		{
			if(anr().getCurrentTransform()->isFondamentalNote(i))	glColor3f(0.0, 0.0, 0.5);
			else													glColor3f(0.4, 0.4, 0.5);
			int x = int(i*step);
			int y = int( (scale_factor*anr().getCurrentTransform()->getComponents()[i]) * (height()-scale_height)) + scale_height;
//			cerr<<"comp="<<anr().getComponents()[i]<<" y="<<y<<endl;
			if(y>0)
			{
				glVertex2i(x, scale_height);
				glVertex2i(x, y);
				glVertex2i(int((i+1)*step)-space, y);
				glVertex2i(int((i+1)*step)-space, scale_height);
			}
		}
		glEnd();

		// tresholds
		glBegin(GL_LINES);
		glColor3f(1,0,0);
		int y = 0;
		
		if(setting_autoScale->isOn())
			y = int(anr().getCurrentTransform()->getComponentsTreshold()*(height()-scale_height)) + scale_height;
		else
			y = int(anr().getCurrentTransform()->getComponentsMax()*anr().getCurrentTransform()->getComponentsTreshold()*(height()-scale_height)) + scale_height;

		glVertex2i(0, y);	glVertex2i(width(), y);
		//	glColor3f(0,1,0);
		//	y = int(anr().getComponentsMax()*anr().m_components_number_treshold*scale_factor*(height()-scale_height)) + scale_height;
		//	glVertex2i(0, y);	glVertex2i(width(), y);
		//	glColor3f(0,0,1);
		//	y = int(anr().getDominantTreshold()*anr().getComponentsMax()*scale_factor*(height()-scale_height)) + scale_height;
		//	glVertex2i(0, y);	glVertex2i(width(), y);
		glEnd();
	}

	// scale
	for(int i=0; i<GetNbSemitones(); i++)
	{
//		string str = StringAddons::toString(i);
		string str = StringAddons::toString(int(i+GetSemitoneMin()));

		if(m_selected_st==i+GetSemitoneMin())
			glColor3f(0,0,1.0);
		else			
			glColor3f(0.5,0.5,0.5);

		glRasterPos2i(int(i*step), 2);

		for(size_t i = 0; i < str.size(); i++)
			glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10, str[i]);
	}

	glFlush();

//	cerr << "/GLComponents::paintGL" << endl;
}

void GLComponents::resizeGL( int w, int h )
{
	// Set the new viewport size
	glViewport(0, 0, (GLint)w, (GLint)h);

	// Choose the projection matrix to be the matrix
	// manipulated by the following calls
	glMatrixMode(GL_PROJECTION);

	// Set the projection matrix to be the identity matrix
	glLoadIdentity();

	// Define the dimensions of the Orthographic Viewing Volume
	glOrtho(0.0, w, 0.0, h, 0.0, 1.0);

	// Choose the modelview matrix to be the matrix
	// manipulated by further calls
	glMatrixMode(GL_MODELVIEW);
}

