// Copyright 2005 "Gilles Degottex"

// This file is part of "Music"

// "Music" is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// "Music" is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "ANR.h"

#include "GLMonoGraph.h"

#include <iostream>
using namespace std;
#include <GL/glut.h>
#include <Music/Music.h>
using namespace Music;

GLMonoGraph::GLMonoGraph(QWidget* parent, const char* name)
: GLRTGraph(name, parent)
, m_name_pos(0)
{
	m_selected_note = NULL;
}

void GLMonoGraph::noteStarted(int tag, int ht, double dt)
{
	NoteView n;
	n.note = (*anr().m_notes_tag2descr.find(tag)).second;
	n.name_pos = m_name_pos;
	m_notes_history.push_front(n);
}
void GLMonoGraph::noteFinished(int tag, int ht, double dt)
{
	m_name_pos++;
	m_name_pos %= 3;
}

void GLMonoGraph::mouseReleaseEvent(QMouseEvent* e)
{
	GLGraph::mouseReleaseEvent(e);

	ANR::NoteDescription* old_selected_note = m_selected_note;

	m_selected_note = NULL;

	double current_time = anr().getTime();
	int used_width = ((setting_spinDuration->value()*anr().m_capture_thread.getSamplingRate())/1000)/m_skip;
	double fw = double(used_width) / setting_spinDuration->value();
	for(size_t i=0; i<m_notes_history.size(); i++)
	{
		int left = width() - int(fw * (current_time-m_notes_history[i].note->start_time));
		int right=0;
		if(m_notes_history[i].note->finished)
			right = left + int(fw * m_notes_history[i].note->duration);
		else
			right = width();

		if(e->x()>=left && e->x()<=right)
			m_selected_note = m_notes_history[i].note;
	}

	if(old_selected_note!=m_selected_note)
	{
		if(m_selected_note!=NULL)
			emit(selectedNoteChanged(m_selected_note->tag));
		else
			emit(selectedNoteChanged(0));
	}

	updateGL();
}

void GLMonoGraph::paintGL()
{
	if(GetSamplingRate()<=0)	return;

	glClear(GL_COLOR_BUFFER_BIT);

	base_paint();

	// paint the recognized notes
	double current_time = anr().getTime();
	while(!m_notes_history.empty() && anr().isRunning() &&
			current_time-m_notes_history.back().note->start_time > setting_spinDuration->value())
		m_notes_history.pop_back();
	double fw = float(width()) / setting_spinDuration->value();
	for(size_t i=0; i<m_notes_history.size(); i++)
	{
		int left = width() - int(fw * (current_time-m_notes_history[i].note->start_time));
		int right=0;
		if(m_notes_history[i].note->finished)
			right = left + int(fw * m_notes_history[i].note->duration);
		else
			right = width();
//right -= int(fw * anr().m_quantizer.getTolerance()*anr().getWindowDuration(m_notes_history[i].ht));
//right -= int(fw * anr().m_quantizer.getTolerance());

		float contrast = 0.0;
		if(m_notes_history[i].note==m_selected_note)
			contrast = 0.1;

		glColor4f(0.90-contrast, 0.90-contrast, 0.97, 0.5);
		glBegin(GL_QUADS);
		glVertex2i(left, height());	glVertex2i(right, height());
		glVertex2i(right, 0);		glVertex2i(left, 0);
		glEnd();
		glBegin(GL_LINES);
		glColor3f(0.7-contrast, 0.7-contrast, 0.8);
		glVertex2i(left, 0); glVertex2i(left, height());
		glVertex2i(right, 0); glVertex2i(right, height());
		glEnd();

		glColor3f(0.2-contrast,0.2-contrast,1.0);
		string str = h2n(m_notes_history[i].note->ht);
		glRasterPos2i(left+3, 16*m_notes_history[i].name_pos+3);
		for(size_t i = 0; i < str.size(); i++)
			glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, str[i]);
	}

	glFlush();
}

