// Copyright 2005 "Gilles Degottex"

// This file is part of "Music"

// "Music" is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// "Music" is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "ANR.h"

#include "GLPolyGraph.h"

#include <iostream>
using namespace std;
#include <GL/glut.h>
#include <Music/Music.h>
using namespace Music;

GLPolyGraph::GLPolyGraph(QWidget* parent, const char* name)
: GLRTGraph(name, parent)
{
}

void GLPolyGraph::noteStarted(int tag, int ht, double dt)
{
	//  	cout << "ANR::noteStarted" << endl;
}
void GLPolyGraph::noteFinished(int tag, int ht, double dt)
{
	// 	cout << "ANR::noteFinished" << endl;
}

void GLPolyGraph::paintGL()
{
	if(GetSamplingRate()<=0)	return;

	glClear(GL_COLOR_BUFFER_BIT);

	if(!anr().m_queue.empty())
	{
		base_paint(0.75);

		// paint grid
		float step = float(height())/GetNbSemitones();
		glBegin(GL_LINES);
		glColor4f(0.0, 0.0, 0.0, 0.15);
		for(size_t i=0; i<GetNbSemitones(); i++)
		{
			int y = int(i*step)-1;
			glVertex2i(0, y);
			glVertex2i(width(), y);
		}
		glEnd();

//		int max_size = (setting_spinDuration->value()*GetSamplingRate())/1000;
//		m_skip = max_size/width();
//		if(m_skip<1)	m_skip = 1;

		// paint the recognized notes
		double current_time = anr().getTime();
		int used_width = ((setting_spinDuration->value()*GetSamplingRate())/1000)/m_skip;
		double fw = double(used_width) / setting_spinDuration->value();
		int space = (step>2)?2:0;
		for(int ht=GetSemitoneMin(); ht<=GetSemitoneMax(); ht++)
		{
			int rel_ht = ht-anr().m_quantizer.getSemitoneMin();
			for(size_t i=0; i<anr().m_notes_history[rel_ht].size() &&
					current_time-anr().m_notes_history[rel_ht][i]->start_time<setting_spinDuration->value(); i++)
			{
				ANR::NoteDescription* note = anr().m_notes_history[rel_ht][i];
				int left = width() - int(fw * (current_time-note->start_time));
				int right=0;
				if(note->finished)
					right = left + int(fw * note->duration);
				else
					right = width();
//				right -= int(fw * anr().m_quantizer.getTolerance()*anr().getWindowDuration(m_notes_history[i].ht));

				int hi = note->ht - GetSemitoneMin();
				int bottom = int(hi*step);
				int top = int((hi+1)*step)-space;

				glColor4f(0.5, 0.5, 1.0, 0.75);
				glBegin(GL_QUADS);
				glVertex2i(left, top);		glVertex2i(right, top);
				glVertex2i(right, bottom);	glVertex2i(left, bottom);
				glEnd();
				glBegin(GL_LINES);
				glColor3f(0.5, 0.5, 1.0);
				glVertex2i(left, top); glVertex2i(right, top);
				glVertex2i(right, top+1); glVertex2i(right, bottom);
				glVertex2i(right, bottom); glVertex2i(left, bottom);
				glVertex2i(left, bottom); glVertex2i(left, top+1);
				glEnd();
			}
		}
	}

	glFlush();
}

