// Copyright 2005 "Gilles Degottex"

// This file is part of "Music"

// "Music" is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// "Music" is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "GLRTGraph.h"

#include <iostream>
using namespace std;
#include <qtooltip.h>
#include <qimage.h>
#include <GL/glut.h>
#include <Music/Music.h>
using namespace Music;

#include "ANR.h"

GLRTGraph::GLRTGraph(const QString& name, QWidget* parent)
: GLGraph(name, parent)
{
	// settings

//	QImage img;
//	img.loadFromData(g_icon_graph, sizeof(g_icon_graph), "PNG");
	setting_showStabilityGraph = new QAction(this);
	setting_showStabilityGraph->setMenuText(tr("Show Stability graph"));
//	setting_showStabilityGraph->setIconSet(QIconSet(QImage(img)));
//	QToolTip::add(setting_showStabilityGraph, tr("Max height"));
	setting_showStabilityGraph->setToggleAction(true);
	setting_showStabilityGraph->setOn(true);
	connect(setting_showStabilityGraph, SIGNAL(toggled(bool)), this, SLOT(update()));
	setting_showStabilityGraph->addTo(&m_popup_menu);
}

void GLRTGraph::save()
{
	GLGraph::save();
}
void GLRTGraph::load()
{
	GLGraph::load();
}
void GLRTGraph::clearSettings()
{
	GLGraph::clearSettings();
}

void GLRTGraph::base_paint(float graph_gray)
{
	GLGraph::base_paint(graph_gray);
	
	// paint the algo latency 
	if(anr().getCurrentAlgorithm()!=NULL)
	{
		glColor4f(0.5, 0.5, 0.5, 0.15);
		glBegin(GL_QUADS);
		int x = int(setting_spinDuration->value()-1000.0*anr().getCurrentAlgorithm()->getSampleAlgoLatency()/GetSamplingRate()) * width() / setting_spinDuration->value();
		glVertex2i(x, 0);				glVertex2i(x, height());
		glVertex2i(width(), height());	glVertex2i(width(), 0);
		glEnd();
		glBegin(GL_LINES);
		glColor4f(0.5, 0.5, 0.5, 0.25);
		glVertex2i(x, 0);			glVertex2i(x, height());
		glEnd();
	}
	// paint the quantizer tolerance
	glColor4f(0.5, 0.5, 1.0, 0.5);
	glBegin(GL_QUADS);
	int x = int(setting_spinDuration->value()-anr().m_quantizer.getTolerance()) * width() / setting_spinDuration->value();
	glVertex2i(x, 0);				glVertex2i(x, height());
	glVertex2i(width(), height());	glVertex2i(width(), 0);
	glEnd();
	glBegin(GL_LINES);
	glColor4f(0.5, 0.5, 1.0, 0.75);
	glVertex2i(x, 0);			glVertex2i(x, height());
	glEnd();
	// paint the real refresh time
	glColor4f(1.0, 0.5, 0.5, 0.5);
	glBegin(GL_QUADS);
	x = int(setting_spinDuration->value()-anr().getRefreshTime()) * width() / setting_spinDuration->value();
	glVertex2i(x, 0);				glVertex2i(x, height());
	glVertex2i(width(), height());	glVertex2i(width(), 0);
	glEnd();
	glBegin(GL_LINES);
	glColor4f(1.0, 0.5, 0.5, 0.75);
	glVertex2i(x, 0);			glVertex2i(x, height());
	glEnd();

	if(setting_showStabilityGraph->isOn())
	{
		glColor3f(0.85, 0.5, 0.5);
		glBegin(GL_LINES);
		for(size_t i=0; i<anr().m_recognition_stats.size(); i++)
		{
			int x = int(width() - width() * (anr().getTime()-anr().m_recognition_stats[i].time)/setting_spinDuration->value());
			glVertex2i(x, height());	glVertex2i(x, int(height()-anr().m_recognition_stats[i].refresh));
		}
		glEnd();

		glColor3f(0.0, 0.75, 0.0);
		glBegin(GL_LINES);
		for(size_t i=0; i<anr().m_recognition_stats.size(); i++)
		{
			int x = int(width() - width() * (anr().getTime()-anr().m_recognition_stats[i].time)/setting_spinDuration->value());
			int y = 0;
			glVertex2i(x, y);			glVertex2i(x, y+anr().m_recognition_stats[i].pending_data/100);
		}
		glEnd();

//		glColor3f(0.0, 0.0, 0.75);
//		glBegin(GL_LINES);
//		for(size_t i=0; i<anr().m_recognition_stats.size(); i++)
//		{
//			int x = int(width - width * (anr().getTime()-anr().m_recognition_stats[i].time)/m_graph_duration);
//			int y = height()-100;
//			glVertex2i(x, y);			glVertex2i(x, y+anr().m_recognition_stats[i].used_recon);
//		}
//		glEnd();
	}
}

